@@ -1,4 +1,4 @@
-/*  Copyright (c) 2003-2013 Xfce Development Team
+/*  Copyright (c) 2003-2014 Xfce Development Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,15 @@
 
 #define ROUND_TO_INT(default_format) (round ? "%.0f" : default_format)
 
+/* Converts temperatures in Celcius to Fahrenheit while preventing
+ * negative values rounded to zero from being displayed as "-0 °F". */
+#define CALC_FAHRENHEIT(round, temperature)              \
+do {                                                     \
+    temperature = temperature * 9.0 / 5.0 + 32;          \
+    if (round && temperature > -0.5 && temperature < 0)  \
+        temperature = 0;                                 \
+} while (0)
+
 #define LOCALE_DOUBLE(value, format)                        \
     (value                                                  \
      ? g_strdup_printf(format, g_ascii_strtod(value, NULL)) \
@@ -259,10 +268,55 @@ calc_apparent_temperature(const xml_location *loc,
            Forecasting. 13, 1998, S. 1187–1193 */
         return 1.41 - 1.162 * windspeed + 0.980 * temp
             + 0.0124 * windspeed * windspeed + 0.0185 * windspeed * temp;
+
+    default:
+        return temp;
     }
 }
 
 
+/*
+ * Return wind direction name for wind degrees, which gives the
+ * direction the wind is coming _from_.
+ */
+static gchar*
+wind_dir_name_by_deg(gchar *degrees, gboolean long_name)
+{
+    gdouble deg;
+
+    if (G_UNLIKELY(degrees == NULL))
+        return "";
+
+    deg = string_to_double(degrees, 0);
+
+    if (deg >= 360 - 22.5 || deg < 45 - 22.5)
+        return (long_name) ? _("North") : _("N");
+
+    if (deg >= 45 - 22.5 && deg < 45 + 22.5)
+        return (long_name) ? _("North-East") : _("NE");
+
+    if (deg >= 90 - 22.5 && deg < 90 + 22.5)
+        return (long_name) ? _("East") : _("E");
+
+    if (deg >= 135 - 22.5 && deg < 135 + 22.5)
+        return (long_name) ? _("South-East") : _("SE");
+
+    if (deg >= 180 - 22.5 && deg < 180 + 22.5)
+        return (long_name) ? _("South") : _("S");
+
+    if (deg >= 225 - 22.5 && deg < 225 + 22.5)
+        return (long_name) ? _("South-West") : _("SW");
+
+    if (deg >= 270 - 22.5 && deg < 270 + 22.5)
+        return (long_name) ? _("West") : _("W");
+
+    if (deg >= 315 - 22.5 && deg < 315 + 22.5)
+        return (long_name) ? _("North-West") : _("NW");
+
+    return "";
+}
+
+
 gchar *
 get_data(const xml_time *timeslice,
          const units_config *units,
@@ -299,7 +353,7 @@ get_data(const xml_time *timeslice,
     case TEMPERATURE:      /* source is in °C */
         val = string_to_double(loc->temperature_value, 0);
         if (units->temperature == FAHRENHEIT)
-            val = val * 9.0 / 5.0 + 32.0;
+            CALC_FAHRENHEIT(round, val);
         return g_strdup_printf(ROUND_TO_INT("%.1f"), val);
 
     case PRESSURE:         /* source is in hectopascals */
@@ -340,7 +394,7 @@ get_data(const xml_time *timeslice,
         return g_strdup_printf("%.0f", val);
 
     case WIND_DIRECTION:
-        return CHK_NULL(loc->wind_dir_name);
+        return g_strdup(wind_dir_name_by_deg(loc->wind_dir_deg, FALSE));
 
     case WIND_DIRECTION_DEG:
         return LOCALE_DOUBLE(loc->wind_dir_deg, ROUND_TO_INT("%.1f"));
@@ -353,14 +407,14 @@ get_data(const xml_time *timeslice,
         if (val == INVALID_VALUE)
             return g_strdup("");
         if (units->temperature == FAHRENHEIT)
-            val = val * 9.0 / 5.0 + 32.0;
+            CALC_FAHRENHEIT(round, val);
         return g_strdup_printf(ROUND_TO_INT("%.1f"), val);
 
     case APPARENT_TEMPERATURE:
         val = calc_apparent_temperature(loc, units->apparent_temperature,
                                         night_time);
         if (units->temperature == FAHRENHEIT)
-            val = val * 9.0 / 5.0 + 32.0;
+            CALC_FAHRENHEIT(round, val);
         return g_strdup_printf(ROUND_TO_INT("%.1f"), val);
 
     case CLOUDS_LOW:
@@ -382,10 +436,10 @@ get_data(const xml_time *timeslice,
     case FOG:
         return LOCALE_DOUBLE(loc->fog_percent, ROUND_TO_INT("%.1f"));
 
-    case PRECIPITATIONS:   /* source is in millimeters */
+    case PRECIPITATION:   /* source is in millimeters */
         val = string_to_double(loc->precipitation_value, 0);
 
-        /* For snow, adjust precipitations dependent on temperature. Source:
+        /* For snow, adjust precipitation dependent on temperature. Source:
            http://answers.yahoo.com/question/index?qid=20061230123635AAAdZAe */
         if (loc->symbol_id == SYMBOL_SNOWSUN ||
             loc->symbol_id == SYMBOL_SNOW ||
@@ -405,7 +459,7 @@ get_data(const xml_time *timeslice,
                 val *= 3;
         }
 
-        if (units->precipitations == INCHES) {
+        if (units->precipitation == INCHES) {
             val /= 25.4;
             return g_strdup_printf("%.2f", val);
         } else
@@ -472,8 +526,8 @@ get_unit(const units_config *units,
         /* TRANSLATORS: Percentage sign is used like a unit for
            clouds, fog, humidity */
         return _("%");
-    case PRECIPITATIONS:
-        return (units->precipitations == INCHES) ? _("in") : _("mm");
+    case PRECIPITATION:
+        return (units->precipitation == INCHES) ? _("in") : _("mm");
     case SYMBOL:
     case WIND_BEAUFORT:
     case WIND_DIRECTION:
@@ -523,47 +577,6 @@ is_night_time(const xml_astro *astro)
 }
 
 
-/*
- * Return wind direction name (S, N, W,...) for wind degrees.
- */
-static gchar*
-wind_dir_name_by_deg(gchar *degrees, gboolean long_name)
-{
-    gdouble deg;
-
-    if (G_UNLIKELY(degrees == NULL))
-        return "";
-
-    deg = string_to_double(degrees, 0);
-
-    if (deg >= 360 - 22.5 || deg < 45 - 22.5)
-        return (long_name) ? _("North") : _("N");
-
-    if (deg >= 45 - 22.5 && deg < 45 + 22.5)
-        return (long_name) ? _("North-East") : _("NE");
-
-    if (deg >= 90 - 22.5 && deg < 90 + 22.5)
-        return (long_name) ? _("East") : _("E");
-
-    if (deg >= 135 - 22.5 && deg < 135 + 22.5)
-        return (long_name) ? _("South-East") : _("SE");
-
-    if (deg >= 180 - 22.5 && deg < 180 + 22.5)
-        return (long_name) ? _("South") : _("S");
-
-    if (deg >= 225 - 22.5 && deg < 225 + 22.5)
-        return (long_name) ? _("South-West") : _("SW");
-
-    if (deg >= 270 - 22.5 && deg < 270 + 22.5)
-        return (long_name) ? _("West") : _("W");
-
-    if (deg >= 315 - 22.5 && deg < 315 + 22.5)
-        return (long_name) ? _("North-West") : _("NW");
-
-    return "";
-}
-
-
 static void
 calculate_symbol(xml_time *timeslice,
                  gboolean current_conditions)
@@ -755,6 +768,33 @@ make_combined_timeslice(xml_weather *wd,
 }
 
 
+void
+merge_astro(GArray *astrodata,
+            const xml_astro *astro)
+{
+    xml_astro *old_astro, *new_astro;
+    guint index;
+
+    g_assert(astrodata != NULL);
+    if (G_UNLIKELY(astrodata == NULL))
+        return;
+
+    /* copy astro, as it may be deleted by the calling function */
+    new_astro = xml_astro_copy(astro);
+
+    /* check for and replace existing astrodata of the same date */
+    if ((old_astro = get_astro(astrodata, astro->day, &index))) {
+        xml_astro_free(old_astro);
+        g_array_remove_index(astrodata, index);
+        g_array_insert_val(astrodata, index, new_astro);
+        weather_debug("Replaced existing astrodata at %d.", index);
+    } else {
+        g_array_append_val(astrodata, new_astro);
+        weather_debug("Appended new astrodata to the existing data.");
+    }
+}
+
+
 void
 merge_timeslice(xml_weather *wd,
                 const xml_time *timeslice)
@@ -846,6 +886,52 @@ time_calc_day(const struct tm time_tm,
 }
 
 
+/*
+ * Compare two xml_astro structs using their date (days) field.
+ */
+gint
+xml_astro_compare(gconstpointer a,
+                  gconstpointer b)
+{
+    xml_astro *a1 = *(xml_astro **) a;
+    xml_astro *a2 = *(xml_astro **) b;
+
+    if (G_UNLIKELY(a1 == NULL && a2 == NULL))
+        return 0;
+    if (G_UNLIKELY(a1 == NULL))
+        return 1;
+    if (G_UNLIKELY(a2 == NULL))
+        return -1;
+
+    return (gint) difftime(a2->day, a1->day) * -1;
+}
+
+
+void
+astrodata_clean(GArray *astrodata)
+{
+    xml_astro *astro;
+    time_t now_t = time(NULL);
+    gint i;
+
+    if (G_UNLIKELY(astrodata == NULL))
+        return;
+
+    for (i = 0; i < astrodata->len; i++) {
+        astro = g_array_index(astrodata, xml_astro *, i);
+        if (G_UNLIKELY(astro == NULL))
+            continue;
+        if (difftime(now_t, astro->day) >= 24 * 3600) {
+            weather_debug("Removing expired astrodata:");
+            weather_dump(weather_dump_astro, astro);
+            xml_astro_free(astro);
+            g_array_remove_index(astrodata, i--);
+            weather_debug("Remaining astrodata entries: %d", astrodata->len);
+        }
+    }
+}
+
+
 /*
  * Compare two xml_time structs using their start and end times,
  * returning the result as a qsort()-style comparison function (less
@@ -1067,26 +1153,63 @@ make_current_conditions(xml_weather *wd,
 }
 
 
+/*
+ * Add days to time_t and set the calculated day to midnight.
+ */
+time_t
+day_at_midnight(time_t day_t,
+                const gint add_days)
+{
+    struct tm day_tm;
+
+    day_tm = *localtime(&day_t);
+    day_tm.tm_mday += add_days;
+    day_tm.tm_hour = day_tm.tm_min = day_tm.tm_sec = 0;
+    day_tm.tm_isdst = -1;
+    day_t = mktime(&day_tm);
+    return day_t;
+}
+
+
+/*
+ * Returns astro data for a given day.
+ */
+xml_astro *
+get_astro_data_for_day(const GArray *astrodata,
+                       const gint day)
+{
+    xml_astro *astro;
+    time_t day_t = time(NULL);
+    gint i;
+
+    if (G_UNLIKELY(astrodata == NULL))
+        return NULL;
+
+    day_t = day_at_midnight(day_t, day);
+
+    for (i = 0; i < astrodata->len; i++) {
+        astro = g_array_index(astrodata, xml_astro *, i);
+        if (astro && (difftime(astro->day, day_t) == 0))
+            return astro;
+    }
+
+    return NULL;
+}
+
+
 /*
  * Get all point data relevant for a given day.
  */
 GArray *
 get_point_data_for_day(xml_weather *wd,
-                       int day)
+                       gint day)
 {
     GArray *found;
     xml_time *timeslice;
-    struct tm day_tm;
-    time_t day_t;
+    time_t day_t = time(NULL);
     gint i;
 
-    /* calculate 00:00 for the requested day */
-    time(&day_t);
-    day_tm = *localtime(&day_t);
-    day_tm.tm_mday += day;
-    day_tm.tm_hour = day_tm.tm_min = day_tm.tm_sec = 0;
-    day_tm.tm_isdst = -1;
-    day_t = mktime(&day_tm);
+    day_t = day_at_midnight(day_t, day);
 
     /* loop over weather data and pick relevant point data */
     found = g_array_new(FALSE, TRUE, sizeof(xml_time *));
@@ -1127,7 +1250,7 @@ make_forecast_data(xml_weather *wd,
     xml_time *ts1, *ts2, *interval = NULL;
     struct tm point_tm, start_tm, end_tm, tm1, tm2;
     time_t point_t, start_t, end_t;
-    gint min, max, point, i, j;
+    gint min = 0, max = 0, point = 0, i, j;
 
     g_assert(wd != NULL);
     if (G_UNLIKELY(wd == NULL))
