@@ -1,4 +1,4 @@
-/*  Copyright (c) 2003-2013 Xfce Development Team
+/*  Copyright (c) 2003-2014 Xfce Development Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 
 G_BEGIN_DECLS
 
+const gchar *get_symbol_for_id(gint id);
+
 const gchar *translate_desc(const gchar *desc,
                             gboolean nighttime);
 
@@ -32,8 +34,6 @@ const gchar *translate_moon_phase(const gchar *moon_phase);
 /* these return a newly allocated string, that should be freed */
 gchar *translate_day(gint weekday);
 
-gchar *translate_wind_direction(const gchar *wdir);
-
 G_END_DECLS
 
 #endif
